using System;
namespace symmetry
{
	
	/// <summary> Uses a gravitational force model to act as a circular "wall". Can be used to
	/// construct circles which either attract or repel items.
	/// 
	/// </summary>
	/// <version>  1.0
	/// </version>
	/// <author>  <a href="http://jheer.org">Jeffrey Heer</a> prefuse(AT)jheer.org
	/// </author>
	public class CircularWallForce:AbstractForce
	{
		override public bool ItemForce
		{
			get
			{
				return true;
			}
			//
			
		}
		override protected internal System.String[] ParameterNames
		{
			get
			{
				return pnames;
			}
			//
			
		}
		
		private static System.String[] pnames = new System.String[]{"GravitationalConstant"};
		
		public const float DEFAULT_GRAV_CONSTANT = - 0.1f;
		public const int GRAVITATIONAL_CONST = 0;
		
		private float x, y, r;
		
		public CircularWallForce(float gravConst, float x, float y, float r)
		{
			params_Renamed = new float[]{gravConst};
			this.x = x; this.y = y;
			this.r = r;
		} //
		
		public CircularWallForce(float x, float y, float r):this(DEFAULT_GRAV_CONSTANT, x, y, r)
		{
		} //
		
		/* (non-Javadoc)
		* @see edu.berkeley.guir.prefusex.force.Force#getForce(edu.berkeley.guir.prefusex.force.ForceItem)
		*/
		public override void  getForce(ForceItem item)
		{
			float[] n = item.location;
			float dx = x - n[0];
			float dy = y - n[1];
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			float d = (float) System.Math.Sqrt(dx * dx + dy * dy);
			float dr = r - d;
			float c = dr > 0?- 1:1;
			float v = c * params_Renamed[GRAVITATIONAL_CONST] * item.mass / (dr * dr);
			if (d == 0.0)
			{
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				dx = ((float) SupportClass.Random.NextDouble() - 0.5f) / 50.0f;
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				dy = ((float) SupportClass.Random.NextDouble() - 0.5f) / 50.0f;
				//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
				d = (float) System.Math.Sqrt(dx * dx + dy * dy);
			}
			item.force[0] += v * dx / d;
			item.force[1] += v * dy / d;
			//System.out.println(dx/d+","+dy/d+","+dr+","+v);
		} //
	} // end of class CircularWallForce
}

