using System;
namespace symmetry
{
	
	/// <summary> Represents a function for computing a force in a force simulation.
	/// 
	/// </summary>
	/// <version>  1.0
	/// </version>
	/// <author>  <a href="http://jheer.org">Jeffrey Heer</a> prefuse(AT)jheer.org
	/// </author>
	public interface Force
		{
			/// <summary> Returns the number of parameters (e.g., gravitational constant or
			/// spring force coefficient) affecting this force function. 
			/// </summary>
			/// <returns> the number of parameters
			/// </returns>
			int ParameterCount
			{
				get;
				
			}
			/// <summary> Indicates if this force function will compute forces
			/// on ForceItem instances
			/// </summary>
			/// <returns> true if this force function processes Force instances 
			/// </returns>
			bool ItemForce
			{
				get;
				
			}
			
			/// <summary> Initialize this force function.</summary>
			/// <param name="fsim">the encompassing ForceSimulator
			/// </param>
			void  init(ForceSimulator fsim);
			
			/// <summary> Returns the specified, numbered parameter.</summary>
			/// <param name="i">the index of the parameter to return
			/// </param>
			/// <returns> the parameter value
			/// </returns>
			float getParameter(int i);
			
			/// <summary> Gets the text name of the requested parameter.</summary>
			/// <param name="i">the index of the parameter
			/// </param>
			/// <returns> a String containing the name of this parameter
			/// </returns>
			System.String getParameterName(int i);
			
			/// <summary> Sets the specified parameter value.</summary>
			/// <param name="i">the index of the parameter
			/// </param>
			/// <param name="val">the new value of the parameter
			/// </param>
			void  setParameter(int i, float val);
			
			/// <summary> Indicates if this force function will compute forces
			/// on Spring instances.
			/// </summary>
			/// <returns> true if this force function processes Spring instances 
			/// </returns>
			bool isSpringForce();
			
			/// <summary> Updates the force calculation on the given ForceItem</summary>
			/// <param name="item">the ForceItem on which to compute updated forces
			/// </param>
			void  getForce(ForceItem item);
			
			/// <summary> Updates the force calculation on the given Spring. The ForceItems
			/// attached to Spring will have their force values updated appropriately.
			/// </summary>
			/// <param name="spring">the Spring on which to compute updated forces
			/// </param>
			void  getForce(Spring spring);
		} // end of interface Force
}