using System;
namespace symmetry
{
	
	/// <summary> Represents a constant gravitational force, like the pull of gravity
	/// for an object on the Earth (F = mg). The force experienced by a
	/// given item is calculated as the product of a GravitationalConstant 
	/// parameter and the mass of the item.
	/// 
	/// </summary>
	/// <version>  1.0
	/// </version>
	/// <author>  <a href="http://jheer.org">Jeffrey Heer</a> prefuse(AT)jheer.org
	/// </author>
	public class GravitationalForce:AbstractForce
	{
		override public bool ItemForce
		{
			get
			{
				return true;
			}
			//
			
		}
		override protected internal System.String[] ParameterNames
		{
			get
			{
				return pnames;
			}
			//
			
		}
		
		private static readonly System.String[] pnames = new System.String[]{"GravitationalConstant", "Direction"};
		public const int GRAVITATIONAL_CONST = 0;
		public const int DIRECTION = 1;
		public const float DEFAULT_FORCE_CONSTANT = 1e-4f;
		public const float DEFAULT_DIRECTION = - 90;
		
		public GravitationalForce(float forceConstant, float direction)
		{
			params_Renamed = new float[]{forceConstant, direction};
		} //
		
		public GravitationalForce():this(DEFAULT_FORCE_CONSTANT, DEFAULT_DIRECTION)
		{
		} //
		
		public override void  getForce(ForceItem item)
		{
			
			//UPGRADE_WARNING: Narrowing conversions may produce unexpected results in C#. 'ms-help://MS.VSCC.2003/commoner/redir/redirect.htm?keyword="jlca1042"'
			float theta = (float) (((- params_Renamed[DIRECTION])/(2*Math.PI))*360);
			float coeff = params_Renamed[GRAVITATIONAL_CONST] * item.mass;
			
			item.force[0] = (float) (item.force[0] + System.Math.Cos(theta) * coeff);
			item.force[1] = (float) (item.force[1] + System.Math.Sin(theta) * coeff);
		} //
	} // end of class GravitationalForce
}